@@ -20,7 +20,5 @@ def upgrade():
 
 
 def downgrade():
-    with op.batch_alter_table('ps_contacts') as batch_op:
-        batch_op.drop_column('user_agent')
-    with op.batch_alter_table('ps_endpoints') as batch_op:
-        batch_op.drop_column('message_context')
+    op.drop_column('ps_contacts', 'user_agent')
+    op.drop_column('ps_endpoints', 'message_context')
