@@ -69,7 +69,6 @@ struct ast_channel {
 	struct ast_cdr *cdr;				/*!< Call Detail Record */
 	struct ast_tone_zone *zone;			/*!< Tone zone as set in indications.conf or
 							 *   in the CHANNEL dialplan function */
-	struct ast_channel_monitor *monitor;		/*!< Channel monitoring */
 	ast_callid callid;			/*!< Bound call identifier pointer */
 	struct ao2_container *dialed_causes;		/*!< Contains tech-specific and Asterisk cause data from dialed channels */
 
@@ -154,7 +153,6 @@ struct ast_channel {
 	int rings;					/*!< Number of rings so far */
 	int priority;					/*!< Dialplan: Current extension priority */
 	int amaflags;					/*!< Set BEFORE PBX is started to determine AMA flags */
-	int macropriority;				/*!< Macro: Current non-macro priority. See app_macro.c */
 	enum ast_channel_adsicpe adsicpe;		/*!< Whether or not ADSI is detected on CPE */
 	unsigned int fin;				/*!< Frames in counters. The high bit is a debug mask, so
 							 *   the counter is only in the remaining bits */
@@ -186,8 +184,6 @@ struct ast_channel {
 	char exten[AST_MAX_EXTENSION];			/*!< Dialplan: Current extension number */
 	char lastcontext[AST_MAX_CONTEXT];		/*!< Dialplan: Previous extension context */
 	char lastexten[AST_MAX_EXTENSION];		/*!< Dialplan: Previous extension number */
-	char macrocontext[AST_MAX_CONTEXT];		/*!< Macro: Current non-macro context. See app_macro.c */
-	char macroexten[AST_MAX_EXTENSION];		/*!< Macro: Current non-macro extension. See app_macro.c */
 	char unbridged;							/*!< non-zero if the bridge core needs to re-evaluate the current
 											 bridging technology which is in use by this channel's bridge. */
 	char is_t38_active;						/*!< non-zero if T.38 is active on this channel. */
