@@ -47,6 +47,9 @@
 		</description>
 		<configFile name="pjsip.conf">
 			<configObject name="outbound-publish">
+				<since>
+					<version>13.0.0</version>
+				</since>
 				<synopsis>The configuration for outbound publish</synopsis>
 				<description><para>
 					Publish is <emphasis>COMPLETELY</emphasis> separate from the rest of
@@ -54,9 +57,15 @@
 					setting a <literal>server_uri</literal> and <literal>event</literal>.
 				</para></description>
 				<configOption name="expiration" default="3600">
+					<since>
+						<version>13.0.0</version>
+					</since>
 					<synopsis>Expiration time for publications in seconds</synopsis>
 				</configOption>
 				<configOption name="outbound_auth" default="">
+					<since>
+						<version>13.0.0</version>
+					</since>
 					<synopsis>Authentication object(s) to be used for outbound publishes.</synopsis>
 					<description><para>
 						This is a comma-delimited list of <replaceable>auth</replaceable>
@@ -71,9 +80,15 @@
 					</description>
 				</configOption>
 				<configOption name="outbound_proxy" default="">
+					<since>
+						<version>13.0.0</version>
+					</since>
 					<synopsis>Full SIP URI of the outbound proxy used to send publishes</synopsis>
 				</configOption>
 				<configOption name="server_uri">
+					<since>
+						<version>13.0.0</version>
+					</since>
 					<synopsis>SIP URI of the server and entity to publish to</synopsis>
 					<description><para>
 						This is the URI at which to find the entity and server to send the outbound PUBLISH to.
@@ -81,6 +96,9 @@
 					</para></description>
 				</configOption>
 				<configOption name="from_uri">
+					<since>
+						<version>13.0.0</version>
+					</since>
 					<synopsis>SIP URI to use in the From header</synopsis>
 					<description><para>
 						This is the URI that will be placed into the From header of outgoing PUBLISH
@@ -89,6 +107,9 @@
 					</para></description>
 				</configOption>
 				<configOption name="to_uri">
+					<since>
+						<version>13.0.0</version>
+					</since>
 					<synopsis>SIP URI to use in the To header</synopsis>
 					<description><para>
 						This is the URI that will be placed into the To header of outgoing PUBLISH
@@ -97,12 +118,21 @@
 					</para></description>
 				</configOption>
 				<configOption name="event" default="">
+					<since>
+						<version>13.0.0</version>
+					</since>
 					<synopsis>Event type of the PUBLISH.</synopsis>
 				</configOption>
 				<configOption name="max_auth_attempts" default="5">
+					<since>
+						<version>13.0.0</version>
+					</since>
 					<synopsis>Maximum number of authentication attempts before stopping the publication.</synopsis>
 				</configOption>
 				<configOption name="transport">
+					<since>
+						<version>13.9.0</version>
+					</since>
 					<synopsis>Transport used for outbound publish</synopsis>
 					<description>
 						<note><para>A <replaceable>transport</replaceable> configured in
@@ -110,10 +140,16 @@
 					</description>
 				</configOption>
 				<configOption name="multi_user" default="no">
+					<since>
+						<version>14.0.0</version>
+					</since>
 					<synopsis>Enable multi-user support</synopsis>
 					<description><para>When enabled the user portion of the server uri is replaced by a dynamically created user</para></description>
 				</configOption>
 				<configOption name="type">
+					<since>
+						<version>13.0.0</version>
+					</since>
 					<synopsis>Must be of type 'outbound-publish'.</synopsis>
 				</configOption>
 			</configObject>
@@ -345,7 +381,6 @@ AST_RWLIST_HEAD_STATIC(publisher_handlers, ast_sip_event_publisher_handler);
 static void sub_add_handler(struct ast_sip_event_publisher_handler *handler)
 {
 	AST_RWLIST_INSERT_TAIL(&publisher_handlers, handler, next);
-	ast_module_ref(ast_module_info->self);
 }
 
 static struct ast_sip_event_publisher_handler *find_publisher_handler_for_event_name(const char *event_name)
@@ -363,7 +398,8 @@ static struct ast_sip_event_publisher_handler *find_publisher_handler_for_event_
 /*! \brief Helper function which cancels the refresh timer on a publisher */
 static void cancel_publish_refresh(struct sip_outbound_publisher *publisher)
 {
-	if (pj_timer_heap_cancel(pjsip_endpt_get_timer_heap(ast_sip_get_pjsip_endpoint()), &publisher->timer)) {
+	if (pj_timer_heap_cancel_if_active(pjsip_endpt_get_timer_heap(ast_sip_get_pjsip_endpoint()),
+		&publisher->timer, 0)) {
 		/* The timer was successfully cancelled, drop the refcount of the publisher */
 		ao2_ref(publisher, -1);
 	}
@@ -428,9 +464,11 @@ static void set_transport(struct sip_outbound_publisher *publisher, pjsip_tx_dat
 {
 	if (!ast_strlen_zero(publisher->owner->publish->transport)) {
 		pjsip_tpselector selector = { .type = PJSIP_TPSELECTOR_NONE, };
+
 		ast_sip_set_tpselector_from_transport_name(
 			publisher->owner->publish->transport, &selector);
 		pjsip_tx_data_set_transport(tdata, &selector);
+		ast_sip_tpselector_unref(&selector);
 	}
 }
 
@@ -643,7 +681,6 @@ void ast_sip_unregister_event_publisher_handler(struct ast_sip_event_publisher_h
 	AST_RWLIST_TRAVERSE_SAFE_BEGIN(&publisher_handlers, iter, next) {
 		if (handler == iter) {
 			AST_RWLIST_REMOVE_CURRENT(next);
-			ast_module_unref(ast_module_info->self);
 			break;
 		}
 	}
@@ -1071,7 +1108,7 @@ static struct sip_outbound_publisher *sip_outbound_publisher_alloc(
 		return NULL;
 	}
 
-	if (ast_sip_push_task_synchronous(NULL, sip_outbound_publisher_init, publisher)) {
+	if (ast_sip_push_task_wait_servant(NULL, sip_outbound_publisher_init, publisher)) {
 		ast_log(LOG_ERROR, "Unable to create publisher for outbound publish '%s'\n",
 			ast_sorcery_object_get_id(client->publish));
 		ao2_ref(publisher, -1);
@@ -1436,14 +1473,16 @@ static struct ast_sip_outbound_publish_state *sip_outbound_publish_state_alloc(
 		return NULL;
 	}
 
-	state->client->datastores = ao2_container_alloc(DATASTORE_BUCKETS, datastore_hash, datastore_cmp);
+	state->client->datastores = ao2_container_alloc_hash(AO2_ALLOC_OPT_LOCK_MUTEX, 0,
+		DATASTORE_BUCKETS, datastore_hash, NULL, datastore_cmp);
 	if (!state->client->datastores) {
 		ao2_ref(state, -1);
 		return NULL;
 	}
 
-	state->client->publishers = ao2_container_alloc(DATASTORE_BUCKETS, sip_outbound_publisher_hash_fn,
-							sip_outbound_publisher_cmp_fn);
+	state->client->publishers = ao2_container_alloc_hash(AO2_ALLOC_OPT_LOCK_MUTEX, 0,
+		DATASTORE_BUCKETS,
+		sip_outbound_publisher_hash_fn, NULL, sip_outbound_publisher_cmp_fn);
 	if (!state->client->publishers) {
 		ao2_ref(state, -1);
 		return NULL;
@@ -1515,8 +1554,8 @@ static int current_state_reusable(struct ast_sip_outbound_publish *publish,
 	 */
 	old_publish = current_state->client->publish;
 	current_state->client->publish = publish;
-	if (ast_sip_push_task_synchronous(
-		    NULL, sip_outbound_publisher_reinit_all, current_state->client->publishers)) {
+	if (ast_sip_push_task_wait_servant(NULL, sip_outbound_publisher_reinit_all,
+		current_state->client->publishers)) {
 		/*
 		 * If the state object fails to re-initialize then swap
 		 * the old publish info back in.
@@ -1556,9 +1595,9 @@ static int sip_outbound_publish_apply(const struct ast_sorcery *sorcery, void *o
 	 * object if created/updated, or keep the old object if an error occurs.
 	 */
 	if (!new_states) {
-		new_states = ao2_container_alloc_options(
-			AO2_ALLOC_OPT_LOCK_NOLOCK, DEFAULT_STATE_BUCKETS,
-			outbound_publish_state_hash, outbound_publish_state_cmp);
+		new_states = ao2_container_alloc_hash(
+			AO2_ALLOC_OPT_LOCK_NOLOCK, 0, DEFAULT_STATE_BUCKETS,
+			outbound_publish_state_hash, NULL, outbound_publish_state_cmp);
 
 		if (!new_states) {
 			ast_log(LOG_ERROR, "Unable to allocate new states container\n");
@@ -1643,8 +1682,6 @@ static int unload_module(void)
 
 static int load_module(void)
 {
-	CHECK_PJSIP_MODULE_LOADED();
-
 	/* As of pjproject 2.4.5, PJSIP_MAX_URL_SIZE isn't exposed yet but we try anyway. */
 	ast_pjproject_get_buildopt("PJSIP_MAX_URL_SIZE", "%d", &pjsip_max_url_size);
 
@@ -1702,4 +1739,5 @@ AST_MODULE_INFO(ASTERISK_GPL_KEY, AST_MODFLAG_GLOBAL_SYMBOLS | AST_MODFLAG_LOAD_
 	.reload = reload_module,
 	.unload = unload_module,
 	.load_pri = AST_MODPRI_CHANNEL_DEPEND,
+	.requires = "res_pjproject,res_pjsip",
 );
