@@ -127,6 +127,18 @@ static struct simple_task_data *simple_task_data_alloc(void)
 	return std;
 }
 
+static void simple_task_data_free(struct simple_task_data *std)
+{
+	if (!std) {
+		return;
+	}
+
+	ast_mutex_destroy(&std->lock);
+	ast_cond_destroy(&std->cond);
+
+	ast_free(std);
+}
+
 static int simple_task(void *data)
 {
 	struct simple_task_data *std = data;
@@ -219,7 +231,9 @@ static enum ast_test_result_state wait_for_empty_notice(struct ast_test *test, s
 	}
 
 	if (!tld->empty_notice) {
-		ast_test_status_update(test, "Test listener not notified that threadpool is empty\n");
+		if (test) {
+			ast_test_status_update(test, "Test listener not notified that threadpool is empty\n");
+		}
 		res = AST_TEST_FAIL;
 	}
 
@@ -319,7 +333,9 @@ AST_TEST_DEFINE(threadpool_push)
 		goto end;
 	}
 
-	ast_threadpool_push(pool, simple_task, std);
+	if (ast_threadpool_push(pool, simple_task, std)) {
+		goto end;
+	}
 
 	wait_for_task_pushed(listener);
 
@@ -328,11 +344,118 @@ AST_TEST_DEFINE(threadpool_push)
 end:
 	ast_threadpool_shutdown(pool);
 	ao2_cleanup(listener);
-	ast_free(std);
+	simple_task_data_free(std);
 	ast_free(tld);
 	return res;
 }
 
+struct efficiency_task_data {
+	struct ast_threadpool *pool;
+	int num_tasks_executed;
+	int shutdown;
+};
+
+static int efficiency_task(void *data)
+{
+	struct efficiency_task_data *etd = data;
+
+	if (etd->shutdown) {
+		return 0;
+	}
+
+	ast_atomic_fetchadd_int(&etd->num_tasks_executed, +1);
+
+	if (ast_threadpool_push(etd->pool, efficiency_task, etd)) {
+		return -1;
+	}
+
+	return 0;
+}
+
+static char *handle_cli_threadpool_push_efficiency(struct ast_cli_entry *e, int cmd, struct ast_cli_args *a)
+{
+	struct ast_threadpool *pool = NULL;
+	struct ast_threadpool_listener *listener = NULL;
+	struct test_listener_data *tld = NULL;
+	enum ast_test_result_state res = AST_TEST_FAIL;
+	struct ast_threadpool_options options = {
+		.version = AST_THREADPOOL_OPTIONS_VERSION,
+		.idle_timeout = 0,
+		.auto_increment = 0,
+		.initial_size = 5,
+		.max_size = 5,
+	};
+	struct efficiency_task_data etd = {
+		.pool = NULL,
+		.num_tasks_executed = 0,
+		.shutdown = 0,
+	};
+	struct timeval start;
+	struct timespec end;
+	int i;
+
+	switch (cmd) {
+	case CLI_INIT:
+		e->command = "threadpool push efficiency";
+		e->usage =
+			"Usage: threadpool push efficiency\n"
+			"       Pushes 200 tasks to a threadpool and measures\n"
+			"       the number of tasks executed within 30 seconds.\n";
+		return NULL;
+	case CLI_GENERATE:
+		return NULL;
+	}
+
+	tld = test_alloc();
+	if (!tld) {
+		return CLI_SUCCESS;
+	}
+
+	listener = ast_threadpool_listener_alloc(&test_callbacks, tld);
+	if (!listener) {
+		goto end;
+	}
+
+	pool = ast_threadpool_create("threadpool_push_efficiency", listener, &options);
+	if (!pool) {
+		goto end;
+	}
+
+	etd.pool = pool;
+
+	/* Push in 200 tasks, cause why not */
+	for (i = 0; i < 200; i++) {
+		if (ast_threadpool_push(pool, efficiency_task, &etd)) {
+			goto end;
+		}
+	}
+
+	/* Wait for 30 seconds */
+	start = ast_tvnow();
+	end.tv_sec = start.tv_sec + 30;
+	end.tv_nsec = start.tv_usec * 1000;
+
+	ast_mutex_lock(&tld->lock);
+	while (ast_cond_timedwait(&tld->cond, &tld->lock, &end) != ETIMEDOUT) {
+	}
+	ast_mutex_unlock(&tld->lock);
+
+	/* Give the total tasks executed, and tell each task to not requeue */
+	ast_cli(a->fd, "Total tasks executed in 30 seconds: %d\n", etd.num_tasks_executed);
+	etd.shutdown = 1;
+
+	res = wait_for_empty_notice(NULL, tld);
+	if (res == AST_TEST_FAIL) {
+		goto end;
+	}
+
+end:
+	ast_threadpool_shutdown(pool);
+	ao2_cleanup(listener);
+	ast_free(tld);
+	return CLI_SUCCESS;
+}
+
 AST_TEST_DEFINE(threadpool_initial_threads)
 {
 	struct ast_threadpool *pool = NULL;
@@ -571,6 +694,97 @@ end:
 	return res;
 }
 
+AST_TEST_DEFINE(threadpool_thread_timeout_thrash)
+{
+	struct ast_threadpool *pool = NULL;
+	struct ast_threadpool_listener *listener = NULL;
+	enum ast_test_result_state res = AST_TEST_FAIL;
+	struct test_listener_data *tld = NULL;
+	struct ast_threadpool_options options = {
+		.version = AST_THREADPOOL_OPTIONS_VERSION,
+		.idle_timeout = 1,
+		.auto_increment = 1,
+		.initial_size = 0,
+		.max_size = 1,
+	};
+	int iteration;
+
+	switch (cmd) {
+	case TEST_INIT:
+		info->name = "thread_timeout_thrash";
+		info->category = "/main/threadpool/";
+		info->summary = "Thrash threadpool thread timeout";
+		info->description =
+			"Repeatedly queue a task when a threadpool thread should timeout.";
+		return AST_TEST_NOT_RUN;
+	case TEST_EXECUTE:
+		break;
+	}
+
+	tld = test_alloc();
+	if (!tld) {
+		return AST_TEST_FAIL;
+	}
+
+	listener = ast_threadpool_listener_alloc(&test_callbacks, tld);
+	if (!listener) {
+		goto end;
+	}
+
+	pool = ast_threadpool_create(info->name, listener, &options);
+	if (!pool) {
+		goto end;
+	}
+
+	ast_threadpool_set_size(pool, 1);
+
+	for (iteration = 0; iteration < 30; ++iteration) {
+		struct simple_task_data *std = NULL;
+		struct timeval start = ast_tvnow();
+		struct timespec end = {
+			.tv_sec = start.tv_sec + options.idle_timeout,
+			.tv_nsec = start.tv_usec * 1000
+		};
+
+		std = simple_task_data_alloc();
+		if (!std) {
+			goto end;
+		}
+
+		/* Wait until the threadpool thread should timeout due to being idle */
+		ast_mutex_lock(&tld->lock);
+		while (ast_cond_timedwait(&tld->cond, &tld->lock, &end) != ETIMEDOUT) {
+			/* This purposely left empty as we want to loop waiting for a time out */
+		}
+		ast_mutex_unlock(&tld->lock);
+
+		if (ast_threadpool_push(pool, simple_task, std)) {
+			res = AST_TEST_FAIL;
+		} else {
+			res = wait_for_completion(test, std);
+		}
+
+		simple_task_data_free(std);
+
+		if (res == AST_TEST_FAIL) {
+			goto end;
+		}
+	}
+
+	res = wait_until_thread_state(test, tld, 0, 0);
+	if (res == AST_TEST_FAIL) {
+		goto end;
+	}
+
+	res = listener_check(test, listener, 1, 1, 30, 0, 0, 1);
+
+end:
+	ast_threadpool_shutdown(pool);
+	ao2_cleanup(listener);
+	ast_free(tld);
+	return res;
+}
+
 AST_TEST_DEFINE(threadpool_one_task_one_thread)
 {
 	struct ast_threadpool *pool = NULL;
@@ -618,7 +832,9 @@ AST_TEST_DEFINE(threadpool_one_task_one_thread)
 		goto end;
 	}
 
-	ast_threadpool_push(pool, simple_task, std);
+	if (ast_threadpool_push(pool, simple_task, std)) {
+		goto end;
+	}
 
 	ast_threadpool_set_size(pool, 1);
 
@@ -647,7 +863,7 @@ AST_TEST_DEFINE(threadpool_one_task_one_thread)
 end:
 	ast_threadpool_shutdown(pool);
 	ao2_cleanup(listener);
-	ast_free(std);
+	simple_task_data_free(std);
 	ast_free(tld);
 	return res;
 
@@ -707,7 +923,10 @@ AST_TEST_DEFINE(threadpool_one_thread_one_task)
 		goto end;
 	}
 
-	ast_threadpool_push(pool, simple_task, std);
+	if (ast_threadpool_push(pool, simple_task, std)) {
+		res = AST_TEST_FAIL;
+		goto end;
+	}
 
 	res = wait_for_completion(test, std);
 	if (res == AST_TEST_FAIL) {
@@ -730,7 +949,7 @@ AST_TEST_DEFINE(threadpool_one_thread_one_task)
 end:
 	ast_threadpool_shutdown(pool);
 	ao2_cleanup(listener);
-	ast_free(std);
+	simple_task_data_free(std);
 	ast_free(tld);
 	return res;
 }
@@ -793,9 +1012,18 @@ AST_TEST_DEFINE(threadpool_one_thread_multiple_tasks)
 		goto end;
 	}
 
-	ast_threadpool_push(pool, simple_task, std1);
-	ast_threadpool_push(pool, simple_task, std2);
-	ast_threadpool_push(pool, simple_task, std3);
+	res = AST_TEST_FAIL;
+	if (ast_threadpool_push(pool, simple_task, std1)) {
+		goto end;
+	}
+
+	if (ast_threadpool_push(pool, simple_task, std2)) {
+		goto end;
+	}
+
+	if (ast_threadpool_push(pool, simple_task, std3)) {
+		goto end;
+	}
 
 	res = wait_for_completion(test, std1);
 	if (res == AST_TEST_FAIL) {
@@ -825,13 +1053,48 @@ AST_TEST_DEFINE(threadpool_one_thread_multiple_tasks)
 end:
 	ast_threadpool_shutdown(pool);
 	ao2_cleanup(listener);
-	ast_free(std1);
-	ast_free(std2);
-	ast_free(std3);
+	simple_task_data_free(std1);
+	simple_task_data_free(std2);
+	simple_task_data_free(std3);
 	ast_free(tld);
 	return res;
 }
 
+static enum ast_test_result_state wait_until_thread_state_task_pushed(struct ast_test *test,
+		struct test_listener_data *tld, int num_active, int num_idle, int num_tasks)
+{
+	enum ast_test_result_state res = AST_TEST_PASS;
+	struct timeval start;
+	struct timespec end;
+
+	res = wait_until_thread_state(test, tld, num_active, num_idle);
+	if (res == AST_TEST_FAIL) {
+		return res;
+	}
+
+	start = ast_tvnow();
+	end.tv_sec = start.tv_sec + 5;
+	end.tv_nsec = start.tv_usec * 1000;
+
+	ast_mutex_lock(&tld->lock);
+
+	while (tld->num_tasks != num_tasks) {
+		if (ast_cond_timedwait(&tld->cond, &tld->lock, &end) == ETIMEDOUT) {
+			break;
+		}
+	}
+
+	if (tld->num_tasks != num_tasks) {
+		ast_test_status_update(test, "Number of tasks pushed %d does not match expected %d\n",
+				tld->num_tasks, num_tasks);
+		res = AST_TEST_FAIL;
+	}
+
+	ast_mutex_unlock(&tld->lock);
+
+	return res;
+}
+
 AST_TEST_DEFINE(threadpool_auto_increment)
 {
 	struct ast_threadpool *pool = NULL;
@@ -887,7 +1150,9 @@ AST_TEST_DEFINE(threadpool_auto_increment)
 		goto end;
 	}
 
-	ast_threadpool_push(pool, simple_task, std1);
+	if (ast_threadpool_push(pool, simple_task, std1)) {
+		goto end;
+	}
 
 	/* Pushing the task should result in the threadpool growing
 	 * by three threads. This will allow the task to actually execute
@@ -910,9 +1175,19 @@ AST_TEST_DEFINE(threadpool_auto_increment)
 	/* Now push three tasks into the pool and ensure the pool does not
 	 * grow.
 	 */
-	ast_threadpool_push(pool, simple_task, std2);
-	ast_threadpool_push(pool, simple_task, std3);
-	ast_threadpool_push(pool, simple_task, std4);
+	res = AST_TEST_FAIL;
+
+	if (ast_threadpool_push(pool, simple_task, std2)) {
+		goto end;
+	}
+
+	if (ast_threadpool_push(pool, simple_task, std3)) {
+		goto end;
+	}
+
+	if (ast_threadpool_push(pool, simple_task, std4)) {
+		goto end;
+	}
 
 	res = wait_for_completion(test, std2);
 	if (res == AST_TEST_FAIL) {
@@ -932,19 +1207,18 @@ AST_TEST_DEFINE(threadpool_auto_increment)
 		goto end;
 	}
 
-	res = wait_until_thread_state(test, tld, 0, 3);
+	res = wait_until_thread_state_task_pushed(test, tld, 0, 3, 4);
 	if (res == AST_TEST_FAIL) {
 		goto end;
 	}
-	res = listener_check(test, listener, 1, 0, 4, 0, 3, 1);
 
 end:
 	ast_threadpool_shutdown(pool);
 	ao2_cleanup(listener);
-	ast_free(std1);
-	ast_free(std2);
-	ast_free(std3);
-	ast_free(std4);
+	simple_task_data_free(std1);
+	simple_task_data_free(std2);
+	simple_task_data_free(std3);
+	simple_task_data_free(std4);
 	ast_free(tld);
 	return res;
 }
@@ -998,7 +1272,9 @@ AST_TEST_DEFINE(threadpool_max_size)
 		goto end;
 	}
 
-	ast_threadpool_push(pool, simple_task, std);
+	if (ast_threadpool_push(pool, simple_task, std)) {
+		goto end;
+	}
 
 	res = wait_for_completion(test, std);
 	if (res == AST_TEST_FAIL) {
@@ -1014,7 +1290,7 @@ AST_TEST_DEFINE(threadpool_max_size)
 end:
 	ast_threadpool_shutdown(pool);
 	ao2_cleanup(listener);
-	ast_free(std);
+	simple_task_data_free(std);
 	ast_free(tld);
 	return res;
 }
@@ -1070,7 +1346,9 @@ AST_TEST_DEFINE(threadpool_reactivation)
 		goto end;
 	}
 
-	ast_threadpool_push(pool, simple_task, std1);
+	if (ast_threadpool_push(pool, simple_task, std1)) {
+		goto end;
+	}
 
 	ast_threadpool_set_size(pool, 1);
 
@@ -1095,7 +1373,10 @@ AST_TEST_DEFINE(threadpool_reactivation)
 	}
 
 	/* Now make sure the threadpool reactivates when we add a second task */
-	ast_threadpool_push(pool, simple_task, std2);
+	if (ast_threadpool_push(pool, simple_task, std2)) {
+		res = AST_TEST_FAIL;
+		goto end;
+	}
 
 	res = wait_for_completion(test, std2);
 	if (res == AST_TEST_FAIL) {
@@ -1117,8 +1398,8 @@ AST_TEST_DEFINE(threadpool_reactivation)
 end:
 	ast_threadpool_shutdown(pool);
 	ao2_cleanup(listener);
-	ast_free(std1);
-	ast_free(std2);
+	simple_task_data_free(std1);
+	simple_task_data_free(std2);
 	ast_free(tld);
 	return res;
 
@@ -1146,6 +1427,19 @@ static struct complex_task_data *complex_task_data_alloc(void)
 	return ctd;
 }
 
+static void complex_task_data_free(struct complex_task_data *ctd)
+{
+	if (!ctd) {
+		return;
+	}
+
+	ast_mutex_destroy(&ctd->lock);
+	ast_cond_destroy(&ctd->stall_cond);
+	ast_cond_destroy(&ctd->notify_cond);
+
+	ast_free(ctd);
+}
+
 static int complex_task(void *data)
 {
 	struct complex_task_data *ctd = data;
@@ -1277,8 +1571,13 @@ AST_TEST_DEFINE(threadpool_task_distribution)
 		goto end;
 	}
 
-	ast_threadpool_push(pool, complex_task, ctd1);
-	ast_threadpool_push(pool, complex_task, ctd2);
+	if (ast_threadpool_push(pool, complex_task, ctd1)) {
+		goto end;
+	}
+
+	if (ast_threadpool_push(pool, complex_task, ctd2)) {
+		goto end;
+	}
 
 	ast_threadpool_set_size(pool, 2);
 
@@ -1315,8 +1614,8 @@ AST_TEST_DEFINE(threadpool_task_distribution)
 end:
 	ast_threadpool_shutdown(pool);
 	ao2_cleanup(listener);
-	ast_free(ctd1);
-	ast_free(ctd2);
+	complex_task_data_free(ctd1);
+	complex_task_data_free(ctd2);
 	ast_free(tld);
 	return res;
 }
@@ -1373,8 +1672,13 @@ AST_TEST_DEFINE(threadpool_more_destruction)
 		goto end;
 	}
 
-	ast_threadpool_push(pool, complex_task, ctd1);
-	ast_threadpool_push(pool, complex_task, ctd2);
+	if (ast_threadpool_push(pool, complex_task, ctd1)) {
+		goto end;
+	}
+
+	if (ast_threadpool_push(pool, complex_task, ctd2)) {
+		goto end;
+	}
 
 	ast_threadpool_set_size(pool, 4);
 
@@ -1426,8 +1730,8 @@ AST_TEST_DEFINE(threadpool_more_destruction)
 end:
 	ast_threadpool_shutdown(pool);
 	ao2_cleanup(listener);
-	ast_free(ctd1);
-	ast_free(ctd2);
+	complex_task_data_free(ctd1);
+	complex_task_data_free(ctd2);
 	ast_free(tld);
 	return res;
 }
@@ -1543,12 +1847,153 @@ end:
 	poke_worker(data3);
 	ast_taskprocessor_unreference(uut);
 	ast_threadpool_shutdown(pool);
-	ast_free(data1);
-	ast_free(data2);
-	ast_free(data3);
+	complex_task_data_free(data1);
+	complex_task_data_free(data2);
+	complex_task_data_free(data3);
 	return res;
 }
 
+struct serializer_efficiency_task_data {
+	struct ast_taskprocessor *serializer[2];
+	int *num_tasks_executed;
+	int *shutdown;
+};
+
+static int serializer_efficiency_task(void *data)
+{
+	struct serializer_efficiency_task_data *etd = data;
+	struct ast_taskprocessor *taskprocessor = etd->serializer[0];
+
+	if (*etd->shutdown) {
+		return 0;
+	}
+
+	ast_atomic_fetchadd_int(etd->num_tasks_executed, +1);
+
+	/* We ping pong a task between a pair of taskprocessors to ensure that
+	 * a single taskprocessor does not receive a thread from the threadpool
+	 * exclusively.
+	 */
+	if (taskprocessor == ast_threadpool_serializer_get_current()) {
+		taskprocessor = etd->serializer[1];
+	}
+
+	if (ast_taskprocessor_push(taskprocessor,
+		serializer_efficiency_task, etd)) {
+		return -1;
+	}
+
+	return 0;
+}
+
+static char *handle_cli_threadpool_push_serializer_efficiency(struct ast_cli_entry *e, int cmd,
+	struct ast_cli_args *a)
+{
+	struct ast_threadpool *pool = NULL;
+	struct ast_threadpool_listener *listener = NULL;
+	struct test_listener_data *tld = NULL;
+	enum ast_test_result_state res = AST_TEST_FAIL;
+	struct ast_threadpool_options options = {
+		.version = AST_THREADPOOL_OPTIONS_VERSION,
+		.idle_timeout = 0,
+		.auto_increment = 0,
+		.initial_size = 5,
+		.max_size = 5,
+	};
+	struct serializer_efficiency_task_data etd[200];
+	struct timeval start;
+	struct timespec end;
+	int i;
+	int num_tasks_executed = 0;
+	int shutdown = 0;
+
+	switch (cmd) {
+		case CLI_INIT:
+			e->command = "threadpool push serializer efficiency";
+			e->usage =
+				"Usage: threadpool push serializer efficiency\n"
+				"       Pushes 200 tasks to a threadpool in serializers and measures\n"
+				"       the number of tasks executed within 30 seconds.\n";
+			return NULL;
+		case CLI_GENERATE:
+			return NULL;
+		}
+
+	tld = test_alloc();
+	if (!tld) {
+		return CLI_SUCCESS;
+	}
+
+	memset(&etd, 0, sizeof(etd));
+
+	listener = ast_threadpool_listener_alloc(&test_callbacks, tld);
+	if (!listener) {
+		goto end;
+	}
+
+	pool = ast_threadpool_create("threadpool_push_serializer_efficiency", listener, &options);
+	if (!pool) {
+		goto end;
+	}
+
+	/* We create 400 (200 pairs) of serializers */
+	for (i = 0; i < 200; i++) {
+		char serializer_name[AST_TASKPROCESSOR_MAX_NAME + 1];
+
+		ast_taskprocessor_build_name(serializer_name, sizeof(serializer_name), "serializer%d", i);
+		etd[i].serializer[0] = ast_threadpool_serializer(serializer_name, pool);
+		if (!etd[i].serializer[0]) {
+			goto end;
+		}
+
+		ast_taskprocessor_build_name(serializer_name, sizeof(serializer_name), "serializer%d", i);
+		etd[i].serializer[1] = ast_threadpool_serializer(serializer_name, pool);
+		if (!etd[i].serializer[1]) {
+			goto end;
+		}
+
+		etd[i].num_tasks_executed = &num_tasks_executed;
+		etd[i].shutdown = &shutdown;
+	}
+
+	/* And once created we push in 200 tasks */
+	for (i = 0; i < 200; i++) {
+		if (ast_taskprocessor_push(etd[i].serializer[0], serializer_efficiency_task, &etd[i])) {
+			goto end;
+		}
+	}
+
+	/* Wait for 30 seconds */
+	start = ast_tvnow();
+	end.tv_sec = start.tv_sec + 30;
+	end.tv_nsec = start.tv_usec * 1000;
+
+	ast_mutex_lock(&tld->lock);
+	while (ast_cond_timedwait(&tld->cond, &tld->lock, &end) != ETIMEDOUT) {
+	}
+	ast_mutex_unlock(&tld->lock);
+
+	/* Give the total tasks executed, and tell each task to not requeue */
+	ast_cli(a->fd, "Total tasks executed in 30 seconds: %d\n", num_tasks_executed);
+	shutdown = 1;
+
+	res = wait_for_empty_notice(NULL, tld);
+	if (res == AST_TEST_FAIL) {
+		goto end;
+	}
+
+end:
+	/* We need to unreference each serializer */
+	for (i = 0; i < 200; i++) {
+		ast_taskprocessor_unreference(etd[i].serializer[0]);
+		ast_taskprocessor_unreference(etd[i].serializer[1]);
+	}
+	ast_threadpool_shutdown(pool);
+	ao2_cleanup(listener);
+	ast_free(tld);
+	return CLI_SUCCESS;
+}
+
 AST_TEST_DEFINE(threadpool_serializer_dupe)
 {
 	enum ast_test_result_state res = AST_TEST_FAIL;
@@ -1603,6 +2048,11 @@ end:
 	return res;
 }
 
+static struct ast_cli_entry cli[] = {
+	AST_CLI_DEFINE(handle_cli_threadpool_push_efficiency, "Push tasks to a threadpool and measure efficiency"),
+	AST_CLI_DEFINE(handle_cli_threadpool_push_serializer_efficiency, "Push tasks to a threadpool in serializers and measure efficiency"),
+};
+
 static int unload_module(void)
 {
 	ast_test_unregister(threadpool_push);
@@ -1610,6 +2060,7 @@ static int unload_module(void)
 	ast_test_unregister(threadpool_thread_creation);
 	ast_test_unregister(threadpool_thread_destruction);
 	ast_test_unregister(threadpool_thread_timeout);
+	ast_test_unregister(threadpool_thread_timeout_thrash);
 	ast_test_unregister(threadpool_one_task_one_thread);
 	ast_test_unregister(threadpool_one_thread_one_task);
 	ast_test_unregister(threadpool_one_thread_multiple_tasks);
@@ -1620,16 +2071,19 @@ static int unload_module(void)
 	ast_test_unregister(threadpool_more_destruction);
 	ast_test_unregister(threadpool_serializer);
 	ast_test_unregister(threadpool_serializer_dupe);
+	ast_cli_unregister_multiple(cli, ARRAY_LEN(cli));
 	return 0;
 }
 
 static int load_module(void)
 {
+	ast_cli_register_multiple(cli, ARRAY_LEN(cli));
 	ast_test_register(threadpool_push);
 	ast_test_register(threadpool_initial_threads);
 	ast_test_register(threadpool_thread_creation);
 	ast_test_register(threadpool_thread_destruction);
 	ast_test_register(threadpool_thread_timeout);
+	ast_test_register(threadpool_thread_timeout_thrash);
 	ast_test_register(threadpool_one_task_one_thread);
 	ast_test_register(threadpool_one_thread_one_task);
 	ast_test_register(threadpool_one_thread_multiple_tasks);
